package org.arpodymov.app;

import gnu.io.*;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

/**
 * @author Andrei Podymov <arpodymov@gmail.com>
 */
public class ComConnection {

    /*
     * Enumerations to convert to gnu.io.SerialPort specifications
     */
    public enum DataBits {
        DATABITS_5(5),
        DATABITS_6(6),
        DATABITS_7(7),
        DATABITS_8(8);

        DataBits(int code) {
            this.code = code;
        }

        private int code;

        public int getCode() {
            return code;
        }
    }

    public enum StopBits {
        STOPBITS_1(1),
        STOPBITS_2(2),
        STOPBITS_1_5(3);

        StopBits(int code) {
            this.code = code;
        }

        private int code;

        public int getCode() {
            return code;
        }
    }

    public enum Parity {
        PARITY_NONE (0),
        PARITY_ODD (1),
        PARITY_EVEN(2),
        PARITY_MARK(3),
        PARITY_SPACE(4);

        Parity(int code) {
            this.code = code;
        }

        private int code;

        public int getCode() {
            return code;
        }
    }

    /*
     * Use to create the instance of class with needed serial params
     */
    public static class Builder {

        private String portName;
        private int baudrate;
        private DataBits databits = DataBits.DATABITS_8;
        private StopBits stopBits = StopBits.STOPBITS_1;
        private Parity parity = Parity.PARITY_NONE;
        private boolean rtsEnable = true;
        private boolean dtrEnable = true;

        public Builder(String portName, int baudrate) {
            this.portName = portName;
            this.baudrate = baudrate;
        }

        public Builder setDatabits(DataBits databits) {
            this.databits = databits;
            return this;
        }

        public Builder setStopBits(StopBits stopBits) {
            this.stopBits = stopBits;
            return this;
        }

        public Builder setParity(Parity parity) {
            this.parity = parity;
            return this;
        }

        public Builder setRtsEnable(boolean rtsEnable) {
            this.rtsEnable = rtsEnable;
            return this;
        }

        public Builder setDtrEnable(boolean dtrEnable) {
            this.dtrEnable = dtrEnable;
            return this;
        }

        public ComConnection build() {
            return new ComConnection(portName, baudrate, databits.getCode(), stopBits.getCode(), parity.getCode(), rtsEnable, dtrEnable);
        }
    }

    private static final String DEFAULT_ENCODING = "UTF-8";

    private SerialPort serialPort;
    private InputStream serialInputStream;
    private OutputStream serialOutputStream;

    private String portName;
    private int baudrate;
    private int databits;
    private int stopbits;
    private int parity;
    private boolean rtsEnable;
    private boolean dtrEnable;

    /*
     * Use Builder instead
     */
    private ComConnection(String portName,
                         int baudrate,
                         int databits,
                         int stopbits,
                         int parity,
                         boolean rtsEnable,
                         boolean dtrEnable) {

        this.portName = portName;
        this.baudrate = baudrate;
        this.databits = databits;
        this.stopbits = stopbits;
        this.parity = parity;
        this.rtsEnable = rtsEnable;
        this.dtrEnable = dtrEnable;
    }

    public void connect () throws ComConnectionException {
        CommPortIdentifier portIdentifier;
        try {
            portIdentifier = CommPortIdentifier.getPortIdentifier(portName);
        } catch (NoSuchPortException e) {
            throw new ComConnectionException("No such port found", e);
        }
        if (portIdentifier.isCurrentlyOwned() ) {
            throw new ComConnectionException("Port currently in use");
        }

        CommPort commPort;
        try {
            commPort = portIdentifier.open(this.getClass().getName(),2000);
        } catch (PortInUseException e) {
            throw new ComConnectionException("Port currently in use", e);
        }

        if (commPort instanceof SerialPort) {
            serialPort = (SerialPort) commPort;
            // Setup serial port params
            try {
                serialPort.setSerialPortParams(baudrate, databits, stopbits, parity);
            } catch (UnsupportedCommOperationException e) {
                throw new ComConnectionException(e);
            }
            serialPort.setDTR(dtrEnable);
            serialPort.setRTS(rtsEnable);
            try {
                // Create steams for IO operations
                serialInputStream = serialPort.getInputStream();
                serialOutputStream = serialPort.getOutputStream();
            } catch (IOException e) {
                throw new ComConnectionException("IOException during connect", e);
            }
        }
        else {
            // We are supporting only serial ports in this solution
            throw new ComConnectionException("Port is not an instance of SerialPort");
        }
    }

    public void close() {
        if (serialPort != null) {
            serialPort.removeEventListener();
            serialPort.close();
        }
    }

    public void executeCommand(String cmd) throws ComConnectionException {
        try {
            serialOutputStream.write(cmd.getBytes(DEFAULT_ENCODING));
        } catch (IOException e) {
            throw new ComConnectionException("Execute command error", e);
        }
    }

    public String readCommand() throws ComConnectionException {
        byte[] buffer = new byte[1024];
        StringBuilder result = new StringBuilder();
        try {
            int b;
            // Append to buffer while not empty
            while ((b = serialInputStream.read(buffer)) > 0) {
                result.append(new String(buffer,0, b));
            }
        }
        catch (IOException e ) {
            throw new ComConnectionException("Read command error", e);
        }
        return result.toString();
    }

    public String GetDeviceName()  {
        return serialPort.getName();
    }
}
