﻿using System;
using System.Collections.Generic;
using System.IO.Ports;
using System.Linq;
using System.Management;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace NetSerialWrapper
{
    public class ComConnection
    {
        private SerialPort serialPort;
        private AutoResetEvent receiveNowEvent;

        /* Use builder instead constructor to instantiate an object */
        private ComConnection  (String portName,
                                int baudrate,
                                Parity parity,
                                Handshake handshake,
                                int dataBits,
                                StopBits stopbits,
                                bool rtsEnable,
                                bool dtrEnable,
                                int readtimeOut,
                                int writeTimeout)
        {
            serialPort = new SerialPort();
            Encoding enc = new UTF8Encoding(true, true);
            serialPort.Encoding = enc;
            serialPort.PortName = portName;
            serialPort.BaudRate = baudrate;
            serialPort.Parity = parity;
            serialPort.Handshake = handshake;
            serialPort.DataBits = dataBits;
            serialPort.StopBits = stopbits;
            serialPort.RtsEnable = rtsEnable;
            serialPort.DtrEnable = dtrEnable;
            serialPort.ReadTimeout = readtimeOut;
            serialPort.WriteTimeout = writeTimeout;
        }

        public class Builder
        {        
            private String portName;
            private int baudrate;

            // Optional fields with default values
            private Parity parity = Parity.None;
            private Handshake handshake = Handshake.None;
            private int dataBits = 8;
            private StopBits stopbits = StopBits.One;
            private bool rtsEnable = true;
            private bool dtrEnable = true;
            private int readtimeOut = 2000;
            private int writeTimeout = 2000;

            public Builder(String portName, int baudrate)
            {
                this.portName = portName;
                this.baudrate = baudrate;
            }
        
            public Builder WithParity(Parity parity)
            {
                this.parity = parity;
                return this;
            }

            public Builder WithHandshake(Handshake handshake)
            {
                this.handshake = handshake;
                return this;
            }

            public Builder WithDataBits(int databits)
            {
                this.dataBits = databits;
                return this;
            }

            public Builder WithStopBits(StopBits stopBits)
            {
                this.stopbits = stopBits;
                return this;
            }

            public Builder WithRtsEnabled(bool rtsEnabled)
            {
                this.rtsEnable = rtsEnabled;
                return this;
            }

            public Builder WithDtrEnabled(bool dtrEnabled)
            {
                this.dtrEnable = dtrEnabled;
                return this;
            }

            public Builder WithReadTimeout(int readTimeout)
            {
                this.readtimeOut = readTimeout;
                return this;
            }

            public Builder WithWriteTimeout(int writeTimeout)
            {
                this.writeTimeout = writeTimeout;
                return this;
            }

            public ComConnection Build()
            {
                return new ComConnection(portName, baudrate, parity, handshake, dataBits, stopbits, rtsEnable, dtrEnable, readtimeOut, writeTimeout);
            }
        }

        public bool Connect()
        {
            try
            {
                if (serialPort != null && !serialPort.IsOpen)
                {
                    receiveNowEvent = new AutoResetEvent(false);
                    serialPort.Open();
                    serialPort.DataReceived += SerialPort_DataReceived;
                    return true;
                }
                return false;
            }
            catch (Exception ex)
            {
                throw new ComConnectionException("Unable to connect to port: " + ex.Message);
            }
        }

        public bool Close()
        {
            try
            {
                if (serialPort != null && serialPort.IsOpen)
                {
                    serialPort.Close();
                    return true;
                }
                return false;
            }
            catch (Exception ex)
            {
                throw new ComConnectionException("Unable to close port: " + ex.Message);
            }
        }

        public void ExecuteCommand(string cmd)
        {
            try
            {
                // Discard buffers and exceute command
                serialPort.DiscardOutBuffer();
                serialPort.DiscardInBuffer();
                receiveNowEvent.Reset();           
                serialPort.Write(cmd);                          
            }
            catch (Exception ex)
            {
                throw new ComConnectionException("Exception during ExecuteCommand: " + ex.Message);
            }
        }

        public string ReadCommand(int numLines)
        {
            // Validate numLines for correct input
            int numLinesToRead;
            if (numLines > 0)
            {
                numLinesToRead = numLines;
            }
            else
            {
                numLinesToRead = 1;
            }
            string buffer = string.Empty;
            try
            {
                int readLines = 0;
                do
                {
                    // Read line and wait until the amount of linesRead is equal to expected in numLinesToRead
                    if (receiveNowEvent.WaitOne(serialPort.ReadTimeout, false))
                    {                     
                        buffer += serialPort.ReadExisting();
                        readLines++;
                    }
                } while (readLines < numLinesToRead); 
            }
            catch
            {
                buffer = string.Empty;
            }
            return buffer;
        }

        private void SerialPort_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            try
            {
                if (e.EventType == SerialData.Chars)
                {
                    receiveNowEvent.Set();
                }
            }
            catch (Exception ex)
            {
                throw new ComConnectionException("Exception during data receiving: " + ex.Message);
            }
        }
        public string GetDeviceName()
        {      
            ManagementObjectSearcher manObjSearch = new ManagementObjectSearcher("Select * from Win32_SerialPort");
            ManagementObjectCollection manObjReturn = manObjSearch.Get();
            foreach (ManagementObject manObj in manObjReturn)
            {
                if (manObj["DeviceID"].ToString() == serialPort.PortName)
                {
                    return manObj["Name"].ToString();
                }
            }
            return "Device info of " + serialPort.PortName + " not found";
        }
    }
}
